@@ -1424,7 +1424,7 @@ tweak_statushash(void *baton,
   return SVN_NO_ERROR;
 }
 
-/* Returns the URL for DB, or NULL: */
+/* Returns the URL for DB */
 static const char *
 find_dir_repos_relpath(const struct dir_baton *db, apr_pool_t *pool)
 {
@@ -1441,14 +1441,11 @@ find_dir_repos_relpath(const struct dir_baton *db, apr_pool_t *pool)
       /* Note that status->repos_relpath could be NULL in the case of a missing
        * directory, which means we need to recurse up another level to get
        * a useful relpath. */
-      if (status)
+      if (status && status->repos_relpath)
         return status->repos_relpath;
 
       repos_relpath = find_dir_repos_relpath(pb, pool);
-      if (repos_relpath)
-        return svn_relpath_join(repos_relpath, db->name, pool);
-      else
-        return NULL;
+      return svn_relpath_join(repos_relpath, db->name, pool);
     }
 }
 
@@ -2144,19 +2141,15 @@ close_file(void *file_baton,
           const char *dir_repos_relpath = find_dir_repos_relpath(fb->dir_baton,
                                                                  pool);
 
-          if (dir_repos_relpath)
-            {
-              /* repos_lock still uses the deprecated filesystem absolute path
-                 format */
+          /* repos_lock still uses the deprecated filesystem absolute path
+             format */
+          const char *repos_relpath = svn_relpath_join(dir_repos_relpath,
+                                                       fb->name, pool);
 
-              const char *repos_relpath = svn_relpath_join(dir_repos_relpath,
-                                                           fb->name, pool);
-
-              repos_lock = apr_hash_get(fb->edit_baton->wb.repos_locks,
-                                        svn_fspath__join("/", repos_relpath,
-                                                         pool),
-                                        APR_HASH_KEY_STRING);
-            }
+          repos_lock = apr_hash_get(fb->edit_baton->wb.repos_locks,
+                                    svn_fspath__join("/", repos_relpath,
+                                                     pool),
+                                    APR_HASH_KEY_STRING);
         }
     }
   else
@@ -2363,7 +2356,11 @@ svn_wc__internal_walk_status(svn_wc__db_t *db,
 
   err = read_info(&dir_info, local_abspath, db, scratch_pool, scratch_pool);
 
-  if (!err && dir_info->kind == svn_wc__db_kind_dir)
+  if (!err
+      && dir_info->kind == svn_wc__db_kind_dir
+      && dir_info->status != svn_wc__db_status_not_present
+      && dir_info->status != svn_wc__db_status_excluded
+      && dir_info->status != svn_wc__db_status_server_excluded)
     {
       anchor_abspath = local_abspath;
       target_name = NULL;
@@ -2386,6 +2383,28 @@ svn_wc__internal_walk_status(svn_wc__db_t *db,
   SVN_ERR(svn_io_stat_dirent(&dirent, local_abspath, TRUE,
                              scratch_pool, scratch_pool));
 
+#ifdef HAVE_SYMLINK
+  if (dirent->special && !skip_root)
+    {
+      svn_io_dirent2_t *this_dirent = svn_io_dirent2_dup(dirent,
+                                                         scratch_pool);
+
+      /* We're being pointed to the status root via a symlink.
+       * Get the real node kind and pretend the path is not a symlink.
+       * This prevents send_status_structure() from treating the root
+       * as a directory obstructed by a file. */
+      SVN_ERR(svn_io_check_resolved_path(local_abspath,
+                                         &this_dirent->kind, scratch_pool));
+      this_dirent->special = FALSE;
+      SVN_ERR(send_status_structure(&wb, local_abspath,
+                                    NULL, NULL, NULL,
+                                    dir_info, this_dirent, get_all,
+                                    status_func, status_baton,
+                                    scratch_pool));
+      skip_root = TRUE;
+    }
+#endif
+
   SVN_ERR(get_dir_status(&wb,
                          anchor_abspath,
                          target_name,
